/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: Cardinality.java,v 2f372b8d0c67 2010/04/28 12:56:25 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.observation;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnegative;
import javax.annotation.Nonnull;
import java.io.Serializable;

/*******************************************************************************
 *
 * Represents the cardinality of an
 * observation. It can be a defined number, a close range (min..max) or an open
 * range (>= min, <= max). A cardinality can also be {@link #UNDEFINED}.
 *
 * @author  Fabrizio Giudici
 * @version $Id: Cardinality.java,v 2f372b8d0c67 2010/04/28 12:56:25 fabrizio $
 *
 ******************************************************************************/
public final class Cardinality implements Serializable
  {
    private static final long serialVersionUID = 5491640062623613211L;

    /** Represents an undefined cardinality. */
    public static final Cardinality UNDEFINED = new Cardinality();

    @CheckForNull
    private final int maxValue;

    @CheckForNull
    private final int minValue;

    private final boolean approx;

    /***************************************************************************
     *
     * Creates a cardinality as a precise value.
     *
     * @param  count  the value
     *
     **************************************************************************/
    @Nonnull
    public static Cardinality valueOf (@Nonnegative int count)
      {
        return new Cardinality(count, count, false);
      }

    @Nonnull
    public static Cardinality approxValueOf (@Nonnegative int count)
      {
        return new Cardinality(count, count, true);
      }

    /***************************************************************************
     *
     * Creates a cardinality as a closed range.
     *
     * @param  min  the lower limit of the range
     * @param  max  the upper limit of the range
     *
     **************************************************************************/
    @Nonnull
    public static Cardinality rangeOf (@Nonnegative int min, @Nonnegative int max)
      {
        return new Cardinality(min, max, false);
      }

    /***************************************************************************
     *
     *
     *
     **************************************************************************/
    private Cardinality()
      {
        minValue = maxValue = -1; // flag value for UNDEFINED.
        approx = false;
      }

    /***************************************************************************
     *
     *
     *
     **************************************************************************/
    private Cardinality (final @Nonnegative int minValue,
                         final @Nonnegative int maxValue,
                         final boolean approx)
      {
        this.maxValue = maxValue;
        this.minValue = minValue;
        this.approx = approx;
      }

    /***************************************************************************
     *
     * Returns the upper limit of the interval.
     *
     * @return  the upper limit of the interval
     *
     **************************************************************************/
    public int getMaxValue()
      {
        return maxValue;
      }

    /***************************************************************************
     *
     * Returns the lower limit of the interval.
     *
     * @return  the lower limit of the interval
     *
     *
     **************************************************************************/
    public int getMinValue()
      {
        return minValue;
      }

    public boolean isApproximated()
      {
        return approx;
      }

    /***************************************************************************
     *
     * {@inheritDoc}
     *
     **************************************************************************/
    @Override
    public boolean equals (final @CheckForNull Object object)
      {
        if (object == null)
          {
            return false;
          }

        if (getClass() != object.getClass())
          {
            return false;
          }

        final Cardinality other = (Cardinality) object;

        return (this.maxValue == other.maxValue) && (this.minValue == other.minValue) && (this.approx == other.approx);
      }

    /***************************************************************************
     *
     * {@inheritDoc}
     *
     **************************************************************************/
    @Override
    public int hashCode()
      {
        int hash = 5;
        hash = 97 * hash + maxValue;
        hash = 97 * hash + minValue;
        hash = 97 * hash + (approx ? 0 : 1);

        return hash;
      }

    /***************************************************************************
     *
     * {@inheritDoc}
     *
     **************************************************************************/
    @Override
    public String toString()
      {
        if (minValue < 0)
          {
            return "undefined";
          }
        else
          {
            return minValue == maxValue ?
                ((approx ? "\u007E":  "")  + Integer.toString(minValue))
                : String.format("%d-%d", minValue, maxValue);
          }
      }
  }
